%%% -*- coding: latin-1 -*-

%%% This file is part of RefactorErl.
%%%
%%% RefactorErl is free software: you can redistribute it and/or modify
%%% it under the terms of the GNU Lesser General Public License as published
%%% by the Free Software Foundation, either version 3 of the License, or
%%% (at your option) any later version.
%%%
%%% RefactorErl is distributed in the hope that it will be useful,
%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%% GNU Lesser General Public License for more details.
%%%
%%% You should have received a copy of the GNU Lesser General Public License
%%% along with RefactorErl.  If not, see <http://plc.inf.elte.hu/erlang/>.
%%%
%%% The Original Code is RefactorErl.
%%%
%%% The Initial Developer of the Original Code is Etvs Lornd University.
%%% Portions created  by Etvs Lornd University and ELTE-Soft Ltd.
%%% are Copyright 2007-2013 Etvs Lornd University, ELTE-Soft Ltd.
%%% and Ericsson Hungary. All Rights Reserved.

%%% @doc Record properties and record based queries

-module(reflib_record).
-vsn("$Rev: 9568 $ ").

%% =============================================================================
%% Exports

-export([find/1, fields/0, field/1, name/1, file/0, form/0, references/0]).

-include("lib.hrl").

%% =============================================================================


%% @spec find(atom()) -> query(#file{}, #record{})
%% @doc The result query returns the record with name `Name'
find(Name) ->
    [{record, {name, '==', Name}}].


%% @spec fields() -> query(#record{}, #field{})
%% @doc The result query returns the fields of the record
fields() ->
    [field].


%% @spec field(atom()) -> query(#record{}, #field{})
%% @doc The result query returns the record's field `Field'
field(Field) ->
    [{field, {name, '==', Field}}].


%% @spec name(node(#record{})) -> atom()
%% @doc The name of the record object
name(Record) ->
    (?Graph:data(Record))#record.name.

%% @spec file() -> query(#record{}, #file{})
%% @doc The result query returns the file that defines the record
file() ->
    [{record, back}].

%% @spec form() -> query(#record{}, #form{})
%% @doc The result query returns the form that defines the record
form() ->
    [{recdef, back}].

%% @spec references() -> query(#record{}, #expression{})
%% @doc The result query returns every reference to the record
references() ->
    [{recref, back}].
