;; This file is part of RefactorErl.
;;
;; RefactorErl is free software: you can redistribute it and/or modify
;; it under the terms of the GNU Lesser General Public License as published
;; by the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; RefactorErl is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU Lesser General Public License for more details.
;;
;; You should have received a copy of the GNU Lesser General Public License
;; along with RefactorErl.  If not, see <http://plc.inf.elte.hu/erlang/>.
;;
;; The Original Code is RefactorErl.
;;
;; The Initial Developer of the Original Code is Eötvös Loránd University.
;; Portions created  by Eötvös Loránd University and ELTE-Soft Ltd.
;; are Copyright 2007-2013 Eötvös Loránd University, ELTE-Soft Ltd.
;; and Ericsson Hungary. All Rights Reserved.

(provide 'refactorerl-depanal)

(require 'cl)
(require 'widget)

(eval-when-compile
  (require 'wid-edit))

(defvar levelvar "mod")
(defvar typevar "all")
(defvar otpvar :true)

(defvar gnodevar "")
(defvar exceptionvar "")
(defvar leavesvar "")
(defvar dotvar "")

(defun refactorerl-depanal-funmod ()
  "Widget : Analise function and module dependencies"
  (interactive)
  (switch-to-buffer "*Function and module dependencies*")
  (kill-all-local-variables)
  (make-local-variable 'levelvar)
  (make-local-variable 'typevar)
  (make-local-variable 'otpvar)
  (make-local-variable 'gnodevar)
  (make-local-variable 'exceptionvar)
  (make-local-variable 'leavesvar)          
  (make-local-variable 'dotvar)   
       
  (let ((inhibit-read-only t))
    (erase-buffer))
  (let ((all (overlay-lists)))
    ;; Delete all the overlays.
    (mapcar 'delete-overlay (car all))
    (mapcar 'delete-overlay (cdr all)))
  (widget-insert " * Function and module dependencies *\n\n")
  ;; Level
  (widget-insert " Level: \t ")
  (widget-create 'radio-button-choice
			 :value "mod"
			 :notify (lambda (widget &rest ignore) 
				   (setq levelvar (widget-value widget))
				   (widget-setup))
			 '(choice-item :format " Mod \t " "mod") '(choice-item :format " Func\n" "func"))
  (widget-insert "\n")
    ;; Type
  (widget-insert " Type:  \t ")
  (widget-create 'radio-button-choice
			 :value "all"
			 :notify (lambda (widget &rest ignore) 
				   (setq typevar (widget-value widget))
				   (widget-setup))
			 '(choice-item :format " All \t " "all") '(choice-item :format " Cycles\n" "cycles"))
  (widget-insert "\n")
    ;; Otp
  (widget-insert " Otp:   \t ")
  (widget-create 'radio-button-choice
			 :value :true
			 :notify (lambda (widget &rest ignore) 
				   (setq otpvar (widget-value widget))
				   (widget-setup))
			 '(choice-item :format "True \t " :true) '(choice-item :format "False\n" :false))
  (widget-insert "\n\n")
  	;;
  (widget-insert " Node options\n\n  Nodes should be specified with graph nodes or with identifier\n  modules can be specified with their names as atoms (e.g. 'mnesia'),\n  while functions are specified by their MFA descriptor as a string\n (e.g. \"io:format/2\")\n\n")
  	;; Gnode
    ;; List of entity or entities that should be the starting point of the analysis
  (widget-insert " List of entity or entities that should be the starting point of the analysis\n ")
  (widget-create 'editable-field
		 :size 13
		 :value ""
		 :notify (lambda (widget &rest ignore) 
				   (setq gnodevar (widget-value widget))
				   (widget-setup))
		 "")
  (widget-insert "\n\n")
  	;; Exception
    ;; List of entities excluded from the analysis
  (widget-insert " List of entities excluded from the analysis\n ")
  (widget-create 'editable-field
		 :size 13
		 :value ""
		 :notify (lambda (widget &rest ignore) 
				   (setq exceptionvar (widget-value widget))
				   (widget-setup))
		 "")
  (widget-insert "\n\n")
  	;; Leaves
    ;; List of those entities which should be included in the analysis, but their children should not
  (widget-insert " List of those entities which should be included in the analysis,\n but their children should not\n ")
  (widget-create 'editable-field
		 :size 13
		 :value ""
		 :notify (lambda (widget &rest ignore) 
				   (setq leavesvar (widget-value widget))
				   (widget-setup))
		 "")
  (widget-insert "\n\n")
  	;; Dot
    ;; The file path of the generated .dot graph description
  (widget-insert " The file path of the generated .dot graph description\n ")
  (widget-create 'editable-field
		 :size 13
		 :value ""
	     :notify (lambda (widget &rest ignore) 
				   (setq dotvar (widget-value widget))
				   (widget-setup))
		 "")
  (widget-insert "\n\n")
  	;;
  (widget-create 'push-button
		 :notify (lambda (&rest ignore) 
                    (if (not (equal dotvar ""))
                        (progn
				            (message "Analysis in progress")
			                (funmod-refac-call levelvar typevar otpvar gnodevar 
                                                    exceptionvar leavesvar dotvar))
                        (error "Please give a proper path for dot files!")))
		 "Analise")
  (widget-insert " ")
  (widget-create 'push-button
		 :notify (lambda (&rest ignore) 
			   (kill-this-buffer))
		 "Cancel")
  (widget-insert "\n")
  (use-local-map widget-keymap)
  (widget-setup))
  
(defvar optionvar "draw")
(defvar fblistvar "")
(defvar othervar :true)

(defun refactorerl-depanal-fblock-default ()
  "Widget : Function block dependency analysis"
  (interactive)
  (switch-to-buffer "* Function block dependency analysis *")
  (kill-all-local-variables)
  (make-local-variable 'optionvar)
  (make-local-variable 'fblistvar)
  (make-local-variable 'othervar)
  (let ((inhibit-read-only t))
    (erase-buffer))
  (let ((all (overlay-lists)))
    ;; Delete all the overlays.
    (mapcar 'delete-overlay (car all))
    (mapcar 'delete-overlay (cdr all)))
  (widget-insert " * Function block dependency analysis *\n\n")
  ;; Level
  (widget-insert " Options\n\n ")
  (widget-create 'radio-button-choice
			 :value "draw"
			 :notify (lambda (widget &rest ignore) 
				   (setq optionvar (widget-value widget))
				   (widget-setup))
;			 '(choice-item :format " Displays the relationship between the given\n function block list \n " "get_rel") 
;			 '(choice-item :format " Decides whether there is a connection between\n the two given function blocks \n " "is_rel") 
;			 '(choice-item :format " Checks for cycles in the dependencies between\n the given function block list. \n " "check_cycle") 
			 '(choice-item :format " Prints out the entire graph or creates a\n subgraph drawing from the given function block list \n " "draw") 
			 '(choice-item :format " Prints out a subgraph which contains the\n cycle(s)\n\n" "draw_cycle"))
  (widget-insert "\n")
    ;; Dot
    ;; The file path of the generated .dot graph description
  (widget-insert " Chosen function block lists for further examinations\n ")
    (widget-insert " (Delimiter character: , Example: module01,module02)\n\n ")
  (widget-create 'editable-field
		 :size 13
		 :value ""
	     :notify (lambda (widget &rest ignore) 
				   (setq fblistvar (widget-value widget))
				   (widget-setup))
		 "")
  (widget-insert "\n\n")
    ;; Level
  (widget-insert " Other (take other category into consideration or not?)\n\n ")
  (widget-create 'radio-button-choice
			 :value :true
			 :notify (lambda (widget &rest ignore) 
				   (setq othervar (widget-value widget))
				   (widget-setup))
			 '(choice-item :format " True \t " :true) '(choice-item :format " False\n\n" :false))
  (widget-insert "\n")
  (widget-insert "\n")
  	;; Dot
    ;; The file path of the generated .dot graph description
  (widget-insert " The file path of the generated .dot graph description\n ")
  (widget-create 'editable-field
		 :size 13
		 :value ""
	     :notify (lambda (widget &rest ignore) 
				   (setq dotvar (widget-value widget))
				   (widget-setup))
		 "")
  (widget-insert "\n\n")
  (widget-create 'push-button
		 :notify (lambda (&rest ignore) 
                    (if (or (not (equal dotvar "")) 
                         (and (not (equal optionvar "draw")) 
                              (not (equal optionvar "draw"))))
                        (progn
				            (message "Analysis in progress")
			                (fblock-refac-call optionvar fblistvar othervar dotvar))
                        (error "Please give a proper path for dot files!")))
		 "Analise")
  (widget-insert " ")
  (widget-create 'push-button
		 :notify (lambda (&rest ignore) 
			   (kill-this-buffer))
		 "Cancel")
  (widget-insert "\n")
  (use-local-map widget-keymap)
  (widget-setup))
  
(defun fblock-refac-call (options fblist other dot)
  	"Calls the proper refactorerl function via the emacs server for fblock dep anal"
    (setq properdot (make-proper-dot-path dot))
    (setq properdot-dot (concatenate 'string properdot ".dot"))
    (setq arglist
		(makearglist `( (:gnode ,(to-fblist fblist)) (:dot ,properdot-dot))))
	(cond
;; These functions were removed.
;;		((equal options "get_rel") (fblock-refac-call* 'fb_relations_get_relations fblist other))
;;		((equal options "is_rel") (fblock-refac-call* 'fb_relations_is_relation fblist))
;;		((equal options "check_cycle") (fblock-refac-call* 'fb_relations_check_cycle fblist))		
		((equal options "draw") (depanal-refac-call-and-draw* 'fb_relations_draw arglist properdot "fb_relations"))		
		((equal options "draw_cycle") (depanal-refac-call-and-draw* 'fb_relations_draw_cycles arglist properdot "fb_relations"))									  
))

(defun fblock-refac-call* (referlfun fblist &rest opt)
  	(lexical-let* ((params (if (equal fblist "") nil (split-string fblist)))
                   (fun-and-args (append (list 'referlfun params) opt))
  		(fun
		  	`(refac-send/callbacks ,fun-and-args
			 (:reply (ok status-lines)
				(depanal-window 'ok status-lines)))
				 ))
		(eval fun)
 	)
				 
)  

(defun funmod-refac-call (level type otp gnode exception leaves dot)
  	"Calls the proper refactorerl function via the emacs server for fun/mod dep anal"
    (setq properdot (make-proper-dot-path dot))
    (setq properdot-dot (concatenate 'string properdot ".dot"))
	(setq arglist
		(makearglist `( (:otp ,otp) (:gnode ,gnode) (:exception ,exception) (:leaves ,leaves) (:dot ,properdot-dot))))
  	(cond
  		((equal level "mod") (if (equal type "cycles") (depanal-refac-call-and-draw* 'cyclic_mod_draw_cycles arglist properdot "dep_mod") 
  												(depanal-refac-call-and-draw* 'cyclic_mod_draw arglist properdot "dep_mod")))
  		((equal level "func") (if (equal type "cycles") (depanal-refac-call-and-draw* 'cyclic_fun_draw_cycles arglist properdot "dep_mod") 
  												(depanal-refac-call-and-draw* 'cyclic_fun_draw arglist properdot "dep_mod")))
  		(t (message "some bug %s" level))
  	)
)
  
(defun depanal-refac-call-and-draw* (referlfun arglist dot &rest rst)
  	"Calls the proper refactorerl function for fun/mod anal and shows the result in a buffer and opens the dependency graph"
    (delete-other-windows) 
    (setf reswin (selected-window))
    (setf filewin (split-window-vertically))
  	(let ((arglist
		  	arglist))
 	(refac-send/callbacks (referlfun arglist)
     (:reply (ok status-lines)
     	(progn
     		(select-window reswin)
     	    (switch-to-buffer "Dependency analysis result")
			(let ((inhibit-read-only t))
			   (erase-buffer)
			   (set (make-local-variable 'last-file-dir) ""))
       		(widget-insert "*** Dependency analysis result ***\n\n")
       		(widget-insert (prin1-to-string status-lines))
       		(widget-insert "\n")
       		(use-local-map widget-keymap)
       		(widget-setup)
      	))
         ))
	 (sit-for 1)
     (let* ((dotpath (make-proper-dot-path dot))  
            (cmd (concatenate 'string "dot -Tpdf " dotpath ".dot -o " dotpath ".pdf")))
     	  (progn
;;		  (message (car rst))
          (setq return (equal "" (shell-command cmd)))
     	  (if (or (equal 0 return) (equal "" return) (equal nil return))
              (progn 
                (select-window filewin) 
                (find-file (concatenate 'string dotpath ".pdf")))
              (message "An error occured during generating pdf with dot")))
     )
)
 	   


;; RegExp function blocks
(defvar rgexpvar "")
(defvar typvar :draw)
(defvar rgexptypevar "file")
(defun refactorerl-depanal-fblock-regexp ()
	""
  (interactive)
  (switch-to-buffer "* Function block dependency analysis *")
  (kill-all-local-variables)
  (make-local-variable 'rgexpvar)
  (make-local-variable 'rgexptypevar)
  (make-local-variable 'typvar)
  (let ((inhibit-read-only t))
    (erase-buffer))
  (let ((all (overlay-lists)))
    ;; Delete all the overlays.
    (mapcar 'delete-overlay (car all))
    (mapcar 'delete-overlay (cdr all)))
  (widget-insert "*** Defining function blocks with regular expressions ***\n\n")
  (setf typvar :draw)
;  (widget-insert " Type\n\n ")
;  (widget-create 'radio-button-choice
;			 :value :get_rel
;			 :notify (lambda (widget &rest ignore) 
;				   (setq typvar (widget-value widget))
;				   (widget-setup))
;			 '(choice-item :format " Decides whether there is a connection between\n the two given function blocks \n " :get_rel) 
;			 '(choice-item :format " Prints out every function block which matches\n the basic regular expression. \n " :list) 
;			 '(choice-item :format " Checks for cycles in the dependencies between\n the given function block list. \n " :cycle) 
;			 '(choice-item :format " Prints out the entire graph or creates a\n subgraph drawing from the given function block list. Output\n file is fb_relations.dot \n " :draw) )

  (widget-insert " Regexp\n\n ")
  (widget-insert " Value type:\n\n ")
  (widget-create 'radio-button-choice
			 :value "file"
			 :notify (lambda (widget &rest ignore) 
				   (setq rgexptypevar (widget-value widget))
				   (widget-setup))
			 '(choice-item :format " A file (its filename) \n " "file") 
			 '(choice-item :format " Regular expression list separated with commas \n " 
                                    "regexp"))
  (widget-insert " Value:\n ")
  (widget-create 'editable-field
		 :size 13
		 :value ""
	     :notify (lambda (widget &rest ignore) 
				   (setq rgexpvar (widget-value widget))
				   (widget-setup))
		 "")
  (widget-insert "\n\n")
  (widget-insert " The file path of the generated .dot graph description\n ")
  (widget-create 'editable-field
		 :size 13
		 :value ""
	     :notify (lambda (widget &rest ignore) 
				   (setq dotvar (widget-value widget))
				   (widget-setup))
		 "")
    (widget-insert "\n\n")
  (widget-create 'push-button
		 :notify (lambda (&rest ignore) 
                    (if (or (not (equal dotvar "")) 
                         (and (not (equal optionvar "draw")) 
                              (not (equal optionvar "draw"))))
                        (progn
				            (message "Analysis in progress")
			                (fblock-regexp-refac-call rgexpvar rgexptypevar 
                                                              typvar dotvar))
                        (error "Please give a proper path for dot files!")))
		 "Analise")
  (widget-insert " ")
  (widget-create 'push-button
		 :notify (lambda (&rest ignore) 
			   (kill-this-buffer))
		 "Cancel")
  (widget-insert "\n")
  (use-local-map widget-keymap)
  (widget-setup)
)

(defun fblock-regexp-refac-call (regexp regexptype type dot)
  "Makes an argument list and calls the proper function for regexp fblock depanal"
  (setq properdot (make-proper-dot-path dot))
  (setq properdot-dot (concatenate 'string properdot ".dot"))
  (let* ((value (if (equal regexptype "regexp")
					(to-fblist regexp)  ;; regexps separated with ,
					regexp))            ;; a single file
		 (arglist (makearglist `((:dot ,properdot-dot) (:type ,type) (:regexp ,value))))
			)
  (progn
	(if (equal type :draw)
		(depanal-refac-call-and-draw* 'refusr_fb_regexp_re arglist properdot "fb_relations")
    	(refac-send/callbacks ('refusr_fb_regexp_re arglist) 
			 (:reply (ok status-lines)
			   (depanal-window 'ok status-lines)))))
		)
)


(defun makearglist (lst)
	"Makes an argument list:
	( (1 2) (1 2 3) ) -> ( #(1 2) #(1 3) )"
	(interactive)
    (remove-if-not (lambda (arg) (not (equal arg nil)))
	(loop for elem in lst
		collect (cond ((equal 2 (length elem)) (destructuring-bind (a b) elem (ifnotnil b (vector a b))))
					  ((equal 3 (length elem)) (destructuring-bind (a b c) elem (ifnotnil b (vector a c))))
				)
	))
)

(defun ifnotnil (elem listvar)
  	""
  	(interactive)
  	(if (equal elem "")
  		nil
  	;	(list listvar))
		listvar)
)

(defun depanal-window (status result)
	(delete-other-windows)
	(switch-to-buffer "DepAnal Result")
	(let ((inhibit-read-only t))
		   (erase-buffer)
		   (set (make-local-variable 'last-file-dir) ""))
	(message "Analysis finished")
	(widget-insert "*** Dependency Analisys results ***\n\n")
	(widget-insert (prin1-to-string status-lines))
	(widget-insert "\n")
)

(defun make-proper-dot-path (dot)
    "Give a path for dot files, dot if it is not empty, else the data dir and
     emacs_dep_file in it (without any extension)"
    (if (equal dot "")
        (concatenate 'string refactorerl-data-dir "/emacs_dep_file")
        dot))

(defun to-fblist (string)
    "Split a string and makes a list of function blocks"
    (setq splitted (split-string string "\\,"))
    (if (equal splitted (list ""))
        nil
        splitted))
